@@ -16,7 +16,9 @@ use Log::Log4perl::Config::Watch;
 
 use constant _INTERNAL_DEBUG => 0;
 
-our $CONFIG_FILE_READS = 0;
+our $CONFIG_FILE_READS       = 0;
+our $CONFIG_INTEGRITY_CHECK  = 1;
+our $CONFIG_INTEGRITY_ERROR  = undef;
 
 # How to map lib4j levels to Log::Dispatch levels
 my @LEVEL_MAP_A = qw(
@@ -118,8 +120,11 @@ sub _init {
 
     my $data = config_read($config);
     
-    #use Data::Dumper;
-    #print Data::Dumper::Dumper($data) if _INTERNAL_DEBUG;
+    if(_INTERNAL_DEBUG) {
+        require Data::Dumper;
+        Data::Dumper->import();
+        print Data::Dumper::Dumper($data);
+    }
 
     my @loggers      = ();
     my %filter_names = ();
@@ -209,10 +214,9 @@ sub _init {
             $filter = Log::Log4perl::Filter->new($filter_name, $type);
         } else {
                 # Filter class
-                eval "require $type";
-                if($@) { 
-                    die "$type doesn't exist";
-                }
+                die "Filter class '$type' doesn't exist" unless
+                     Log::Log4perl::Util::module_available($type);
+                eval "require $type" or die "Require of $type failed ($!)";
 
                 # Invoke with all defined parameter
                 # key/values (except the key 'value' which is the entry 
@@ -269,6 +273,24 @@ sub _init {
 
     #now we're done, set up all the output methods (e.g. ->debug('...'))
     Log::Log4perl::Logger::reset_all_output_methods();
+
+    #Run a sanity test on the config not disabled
+    if($Log::Log4perl::Config::CONFIG_INTEGRITY_CHECK and
+       !config_is_sane()) {
+        warn "Log::Log4perl configuration looks suspicious: ",
+             "$CONFIG_INTEGRITY_ERROR";
+    }
+}
+
+##################################################
+sub config_is_sane {
+##################################################
+    if(scalar keys %Log::Log4perl::Logger::APPENDER_BY_NAME == 0) {
+        $CONFIG_INTEGRITY_ERROR = "No appenders defined";
+        return 0;
+    }
+
+    return 1;
 }
 
 ##################################################
@@ -415,34 +437,22 @@ sub add_layout_by_name {
 
     $layout_class =~ s/org.apache.log4j./Log::Log4perl::Layout::/;
 
-    eval {
-        eval "require $layout_class";
-        if($@) {
-            my $old_err = $@;
-            eval "require Log::Log4perl::Layout::$layout_class";
-
-            if($@) {
-                # If it failed again, revert to the old error message
-                $@ = $old_err;
-            } else {
-                # If it succeeded, leave $@ as "", which indicates success
-                # downstream. And, fix the layout name.
-                $layout_class = "Log::Log4perl::Layout::$layout_class";
-            }
+        # Check if we have this layout class
+    if(!Log::Log4perl::Util::module_available($layout_class)) {
+        if(Log::Log4perl::Util::module_available(
+           "Log::Log4perl::Layout::$layout_class")) {
+            # Someone used the layout shortcut, use the fully qualified
+            # module name instead.
+            $layout_class = "Log::Log4perl::Layout::$layout_class";
+        } else {
+            die "ERROR: trying to set layout for $appender_name to " .
+                "'$layout_class' failed";
         }
-        die $@ if $@;
-           # Eval erroneously succeeds on unknown appender classes if
-           # the eval string just consists of valid perl code (e.g. an
-           # appended ';' in $appenderclass variable). Fail if we see
-           # anything in there that can't be class name.
-        die "Unknown layout '$layout_class'" if $layout_class =~ /[^:\w]/;
-    };
-
-    if ($@) {
-        die "ERROR: trying to set layout for $appender_name to " .
-            "'$layout_class' failed\n$@";
     }
 
+    eval "require $layout_class" or 
+        die "Require to $layout_class failed ($!)";
+
     $appender->layout($layout_class->new(
         $data->{appender}->{$appender_name}->{layout},
         ));
@@ -509,13 +519,23 @@ sub config_read {
                                     # of name/value pairs
         @text = map { $_ . '=' . $config->{$_} } keys %{$config};
 
-    } elsif (ref $config) {
+    } elsif (ref $config eq 'SCALAR') {
         @text = split(/\n/,$$config);
 
+    } elsif (ref $config) {
+            # Caller provided a config parser object, which already
+            # knows which file (or DB or whatever) to parse.
+        $data = $config->parse();
+        return $data;
+
     #TBD
     }elsif ($config =~ m|^ldap://|){
-       eval { require Net::LDAP; require Log::Log4perl::Config::LDAPConfigurator; };
-       if ($@){die "Log4perl: missing Net::LDAP needed to parse LDAP urls\n$@\n"}
+       if(! Log::Log4perl::Util::module_available("Net::LDAP")) {
+           die "Log4perl: missing Net::LDAP needed to parse LDAP urls\n$@\n";
+       }
+
+       require Net::LDAP;
+       require Log::Log4perl::Config::LDAPConfigurator;
 
        return Log::Log4perl::Config::LDAPConfigurator::parse($config);
 
@@ -524,21 +544,19 @@ sub config_read {
         if ($config =~ /^(https?|ftp|wais|gopher|file):/){
             my ($result, $ua);
     
-            eval {
-                require LWP::UserAgent;
-                unless (defined $LWP_USER_AGENT) {
-                    $LWP_USER_AGENT = LWP::UserAgent->new;
-    
-                    # Load proxy settings from environment variables, i.e.:
-                    # http_proxy, ftp_proxy, no_proxy etc (see LWP::UserAgent)
-                    # You need these to go thru firewalls.
-                    $LWP_USER_AGENT->env_proxy;
-                }
-                $ua = $LWP_USER_AGENT;
-            };
+            die "LWP::UserAgent not available" unless
+                Log::Log4perl::Util::module_available("LWP::UserAgent");
 
-            $@ && die "Log4perl cannot load $config, \n".
-                        "reason: LWP::UserAgent didn't load\nerror: $@";
+            require LWP::UserAgent;
+            unless (defined $LWP_USER_AGENT) {
+                $LWP_USER_AGENT = LWP::UserAgent->new;
+    
+                # Load proxy settings from environment variables, i.e.:
+                # http_proxy, ftp_proxy, no_proxy etc (see LWP::UserAgent)
+                # You need these to go thru firewalls.
+                $LWP_USER_AGENT->env_proxy;
+            }
+            $ua = $LWP_USER_AGENT;
 
             my $req = new HTTP::Request GET => $config;
             my $res = $ua->request($req);
@@ -564,12 +582,19 @@ sub config_read {
     print "Reading $config: [@text]\n" if _INTERNAL_DEBUG;
 
     if ($text[0] =~ /^<\?xml /) {
-        eval { require XML::DOM; require Log::Log4perl::Config::DOMConfigurator; };
-        if ($@){die "Log4perl: missing XML::DOM needed to parse xml config files\n$@\n"}
+
+        die "XML::DOM not available" unless
+                Log::Log4perl::Util::module_available("XML::DOM");
+
+        require XML::DOM; 
+        require Log::Log4perl::Config::DOMConfigurator;
+
         XML::DOM->VERSION($Log::Log4perl::DOM_VERSION_REQUIRED);
-        $data = Log::Log4perl::Config::DOMConfigurator::parse(\@text);
-    }else{
-        $data = Log::Log4perl::Config::PropertyConfigurator::parse(\@text)
+        my $parser = Log::Log4perl::Config::DOMConfigurator->new();
+        $data = $parser->parse(\@text);
+    } else {
+        my $parser = Log::Log4perl::Config::PropertyConfigurator->new();
+        $data = $parser->parse(\@text);
     }
 
     return $data;
@@ -722,11 +747,13 @@ sub vars_shared_with_safe_compartment {
     }
     elsif( @args == 1 ) {
         # return vars for given package
-        return $Log::Log4perl::VARS_SHARED_WITH_SAFE_COMPARTMENT{$args[0]};
+        return $Log::Log4perl::VARS_SHARED_WITH_SAFE_COMPARTMENT{
+               $args[0]};
     }
     elsif( @args == 2 ) {
         # add/replace package/var pair
-        $Log::Log4perl::VARS_SHARED_WITH_SAFE_COMPARTMENT{$args[0]} = $args[1];
+        $Log::Log4perl::VARS_SHARED_WITH_SAFE_COMPARTMENT{
+           $args[0]} = $args[1];
     }
 
     return wantarray ? %Log::Log4perl::VARS_SHARED_WITH_SAFE_COMPARTMENT
@@ -775,7 +802,8 @@ sub allowed_code_ops_convenience_map {
     }
     elsif( @args == 1 ) {
         # return single opcode mask
-        return $Log::Log4perl::ALLOWED_CODE_OPS{$args[0]};
+        return $Log::Log4perl::ALLOWED_CODE_OPS{
+                   $args[0]};
     }
     elsif( @args == 2 ) {
         # make sure the mask is an array ref
@@ -783,7 +811,8 @@ sub allowed_code_ops_convenience_map {
             die "invalid mask (not an array ref) for convenience name '$args[0]'";
         }
         # add name/mask pair
-        $Log::Log4perl::ALLOWED_CODE_OPS{$args[0]} = $args[1];
+        $Log::Log4perl::ALLOWED_CODE_OPS{
+            $args[0]} = $args[1];
     }
 
     return wantarray ? %Log::Log4perl::ALLOWED_CODE_OPS
@@ -801,7 +830,8 @@ sub allow_code {
     }
    
     if(@args) {
-        $Log::Log4perl::ALLOW_CODE_IN_CONFIG_FILE = $args[0];
+        $Log::Log4perl::ALLOW_CODE_IN_CONFIG_FILE = 
+            $args[0];
     }
 
     return $Log::Log4perl::ALLOW_CODE_IN_CONFIG_FILE;
